@@ -2,7 +2,7 @@
 
 from .graph_module import GraphModule, InputNode, build_graph_module
 from .mixed_precision import apply_mp_policy, unwrap_mp_policy
-from .module import Module, PaxKind
+from .module import Module, parameters_method
 from .module_and_value import module_and_value
 from .pure import pure
 from .transforms import (
@@ -10,10 +10,8 @@ from .transforms import (
     enable_train_mode,
     freeze_parameters,
     select_parameters,
-    select_states,
     unfreeze_parameters,
     update_parameters,
-    update_states,
 )
 from .utility_modules import Flattener, LazyModule, ParameterModule, StateModule
 from .utils import assert_structure_equal
